
/**
 * \file DB.hxx describes DB interfaces and query result access interfaces
 * \author Li Li	
 * \date 4/30/08
 */

#ifndef DB_HXX_
#define DB_HXX_
extern "C" {
#include "libpq-fe.h"
}
#include "Metadata.hxx"
#include <string>
#include <vector>
using namespace std;

namespace cqosdb {
	
	/**
	 *  Outcome defines interfaces for accessing returned query results.
	 */
	class Outcome {
		public:
		virtual ~Outcome(){}
		
		/**
			Set query results as data elements of the class
			\param res is the results returned from a query. Its type is decided by the DBMS, e.g., PGresult in PostgreSQL.
			\return Return a success flag.		
		*/
		virtual int setElements(PGresult *res)=0; 
		
		/**
			Get number of rows of returned query results
		*/
		virtual int getNumberOfRows()=0;
		
		/**
			Get number of columns of return query results
		*/		
		virtual int getNumberOfColumns()=0;
		
		/**
			Get an element of long type at a specified location.
			\param row No. of row
			\param column No. of column
			\param res return value of the element at the location
		*/
		virtual int getElement(int row, int column, long *res)=0;
		
		/**
			Get an element of double type at a specified location.
			\param row No. of row
			\param column No. of column
			\param res return value of the element at the location
		*/
		virtual int getElement(int row, int column, double *res)=0;
		
		/**
			Get an element of string type at a specified location.
			\param row No. of row
			\param column No. of column
			\param res return value of the element at the location
		*/
		virtual int getElement(int row, int column, string *res)=0;
		
		/**
			Get an element of double type at a specified location.
			\param row No. of row
			\param name column name
			\param res return value of the element at the location
		*/
		virtual int getElement(int row,string name,double *res)=0;  
		
		/**
			Get an element of long type at a specified location.
			\param row No. of row
			\param name column name
			\param res return value of the element at the location
		*/
		virtual int getElement(int row,string name,long *res)=0;
		
		/**
			Get an element of string type at a specified location.
			\param row No. of row
			\param name column name
			\param res return value of the element at the location
		*/
		virtual int getElement(int row,string name,string *res)=0;
		
		/**
			Clear up elements
		*/
		virtual void clear()=0;		
	};

	/**
	 *  DB interfaces specifies a set of database operations to be supported.
	 */
	class DB {
		public:
		virtual ~DB(){}
		
		/**
		Connect to a database server.
		*/
		virtual bool connect()=0;
		
		/**
		Disconnect to a database server.
		*/
		virtual void disconnect()=0;
		
		/**
		Test if connected to a database.
		*/
		virtual bool isClosed()=0;
		
		/**
		Set up connection information, which typically includes database server name, port number, user authorization etc.
		*/
		virtual void setConnectionInfo(string info)=0;
		
		/**
		Get the connection setup information.
		*/
		virtual string getConnectionInfo()=0;
		
		/**
		Pass an integer parameter to SQL command text
			\param param Index of parameter
			\param argument Value of the parameter 
		*/
		virtual int bindParameters(int param,int argument)=0;
		
		/**
		Pass a long parameter to SQL command text
			\param param Index of parameter
			\param argument Value of the parameter 
		*/
		virtual int bindParameters(int param, long argument)=0;
		
		/**
		Pass a double parameter to SQL command text
			\param param Index of parameter
			\param argument Value of the parameter 
		*/
		virtual int bindParameters(int param, double argument)=0;
		
		/**
		Pass a string parameter to SQL command text
			\param param Index of parameter
			\param argument Value of the parameter 
		*/
		virtual int bindParameters(int param, string argument)=0;
		
		/**
		Bind parameters to SQL command text, the function is usually called immediately before executing a SQL command
		*/
		virtual void bindCommand()=0;
		
		/**
		Reset query statement. Usually called after executing a SQL command
		*/
		virtual void resetQuery()=0;
		
		/**
		Submits a command to the server and waits for the result, with the ability to 
		pass parameters separately from the SQL command text.
			\param commd The SQL command string to be executed. If parameters are used, 
			they are referred to in the command string as $1, $2, etc.
			\param res Returned query result set
		*/
		virtual int executeQuery_bindParam(string& commd, Outcome* res)=0; 
		
		/**
		Submits a command to the server and waits for the result, without passing parameters separately.
			\param commd The SQL command string to be executed.
			\param res Returned query result set
		*/
		virtual int executeQuery(string& commd, Outcome* res)=0;  
		
		/**
		Submits a 'insert' command to the server.
			\param commd The insert SQL command string to be executed.
		*/
		virtual int executeInsert(string& commd)=0;
		
		/**
		Store a parameter into database. Implementation of the function is dependent on the database schema used.
			\param trialID The trial associated with the parameter
			\param interEvt The interval_event associated with the parameter
			\param aParam The parameter to be stored
		*/
		virtual void storeParameter(int trialID, string interEvt, Parameter& aParam)=0;
		
		/**
		Store a set of parameters into database. Implementation of the function is dependent on the database schema used.
			\param trialID The trial associated with the parameter set
			\param interEvt The interval_event associated with the parameter set
			\param aParamSet The parameter set to be stored
		*/
		virtual void storeParameterSet(int trialID, string interEvt, ParameterSet& aParamSet)=0;
			
		/**
		Retrieve trials from database, which satisfies that its parameter set value is within [lower, upper]. Implementation of the function is dependent on the database schema used.
			\param lower Lower bounds of the parameter set,
			\param upper Upper bounds of the parameter set, 
			\param trialIDs Returned trial ids that meet the conditions
		*/
		virtual int getMatchingTrialsBetween(ParameterSet& lower, ParameterSet& upper, Outcome* trialIDs)=0;
		
		/**
		Retrieve trials from database, which satisfies that its parameter set value is within [lower-epsilons, lower+epsilons]. Implementation of the function is dependent on the database schema used.
			\param lower Pivort values of the parameter set,
			\param epsilons Offset values 
			\param trialIDs Returned trial ids that meet the conditions
		*/
		virtual int getMatchingTrials(ParameterSet& lower, vector<double> epsilons, Outcome* trialIDs)=0;
			
		/**
		Retrieve parameter value from database. Implementation of the function is dependent on the database schema used.
			\param trialID The trial associated with the parameter
			\param interEvt The interval_event associated with the parameter
			\param aParam in - set parameter name and type;
						  out - fill in value from database
		*/
		virtual void getParameter(int trialID, string interEvt, Parameter* aParam)=0;
		
		/**
		Retrieve parameter set values from database. Implementation of the function is dependent on the database schema used.
			\param trialID The trial associated with the parameter set
			\param interEvt The interval_event associated with the parameter set
			\param aParamSet in - set parameter set name and type;
								   out - fill in values from database
		*/
		virtual void getParameterSet(int trialID, string interEvt, ParameterSet* aParamSet)=0;	// get parameter set values	
	};
}

#endif


