/**
 * \file MetaDB.hxx implements DB and Outcome interfaces defined in DB.hxx
 * \author Li Li
 * \date 4/30/08
 */

#ifndef METADB_HXX_
#define METADB_HXX_

#include <map>
#include <vector>

#include "DB.hxx"
#include "Metadata.hxx"
#include "Comparator.hxx"
#include "ParseDBConfig.hxx"
#include "Utility.hxx"

#define MAXPARAMCOUNT 100

using namespace std;

namespace cqosdb {
	
	/**
		An implementation of DB iterfaces. Please refer to DB.hxx for details of each function
	*/
	class MetaDB: public virtual DB {
		public:
		MetaDB();
		~MetaDB();
		 bool connect();
		 void disconnect();
		 bool isClosed();
		 void setConnectionInfo(string info);
		 string getConnectionInfo();
		 
		 /**
			Get connection information (including user name, password, etc.) from 
			default PerfDMF configuration file located at ~/.ParaProf/perfdmf.cfg
		 */
		 string getConnectionInfoFromDefaultPerfDMFConfig();
		 
		 /**
			Get connection information (including user name, password, etc.) from 
			user input 'configFileName'. Note the file path is absolute.
		 */
		 string getConnectionInfoFromPerfDMFConfig(string configFileName);
		 
		int bindParameters(int param,int argument);
		int bindParameters(int param, long argument);
		int bindParameters(int param, double argument);
		int bindParameters(int param, string argument);
		void bindCommand();
		void resetQuery();
		int executeQuery_bindParam(string& commd, Outcome* res); 
		int executeQuery(string& commd, Outcome* res);  
		int executeInsert(string& commd);
		
		/**
		Store a DefaultyProperty into database.
		This function assumes that default database schema (meta_dbschema.sql) is used.
			\param trialID The trial associated with the parameter
			\param interEvt The interval_event associated with the parameter
			\param aParam The DefaultProperty object to be stored
		*/
		void storeParameter(int trialID, string interEvt, Parameter& aParam);
		
		/**
		Store a PropertySet into database.
		This function assumes that default database schema (meta_dbschema.sql) is used. 
			\param trialID The trial associated with the parameter set
			\param interEvt The interval_event associated with the parameter set
			\param aParamSet The PropertySet to be stored
		*/
		void storeParameterSet(int trialID, string interEvt, ParameterSet& aParamSet);
			
		/**
		Retrieve trials from database. The trial have parameters of the same name in 'lower' and 'upper', and the parameter values is within [lower, upper].
		For example, lower=[('nsplit', 'double', 23.0), ('split', 'double', 2.0)], and 
					 upper=[('nsplit', 'double', 455.0), ('split', 'double', 677.0)]
		Then the function returns trials whose parameters 'nsplit' and 'split' satisfy:   23.0 <= nsplit <= 455.0 AND 2.0 <= split<= 677.0.
		This function assumes that default database schema (meta_dbschema.sql) is used. 
		*/
		int getMatchingTrialsBetween(ParameterSet& lower, ParameterSet& upper, Outcome* trialIDs);
		
		/**
		Retrieve trials from database. The trials have parameter of the same name in 'lower', and the parameter values is within [lower-epsilons, lower+epsilons].
		For example, lower=[('nsplit', 'double', 23.0), ('split', 'double', 2.0)], and epsilons = [3.0, 56.0],
		Then the function returns trials whose  parameters 'nsplit' and 'split' satisfy:  abs(nsplit-23.0) <= 3.0 AND abs(split-2.0)<= 56.0.
		This function assumes that default database schema (meta_dbschema.sql) is used. 
		*/
		int getMatchingTrials(ParameterSet& lower, vector<double> epsilons, Outcome* trialIDs);
		
		/**
		Retrieve DefaultProperty value from database.
		This function assumes that default database schema (meta_dbschema.sql) is used. 
			\param trialID The trial associated with the parameter
			\param interEvt The interval_event associated with the parameter
			\param aParam in - set parameter name and type;
						  out - fill in value from database
		*/	
		void getParameter(int trialID, string interEvt, Parameter* aParam); 
		
		/**
		Retrieve PropertySet values from database. 
		This function assumes that default database schema (meta_dbschema.sql) is used. 
			\param trialID The trial associated with the parameter set
			\param interEvt The interval_event associated with the parameter set
			\param aParamSet in - set parameter set name and type;
								   out - fill in values from database
		*/
		void getParameterSet(int trialID, string interEvt, ParameterSet* aParamSet);	

		public:

		 /// connection to the server in PostgreSQL
                PGconn *conn;  
	
		private:
		
		/// flag that shows if connected to a database server
		int connected;
		
		/// connection information
		const char* conninfo;
		
		/// count of parameters in a SQL string text
		int paramCount;
		
		/// array of parameters in a SQL string text, which will be used in executeQuery_bindParam().
		const char *paramValues[MAXPARAMCOUNT];
		
		/// a map of parameter index to parameter name.
		map <int, string> stringarg;
		
		//MetaDB &operator=(const MetaDB &other);
	};
	
	/**
		This class defines opertions related to a row of query returned data
	*/
	class ColumnData{ 

		public:
		ColumnData(){};
		~ColumnData();
		
		/**
			Clear up internal data
		*/
		void clear();
		
		/// a map of index to long type elements.
		map <int, long> longarg;
		
		/// a map of index to string type elements
		map <int, string> stringarg;
		
		/// a map of index to double type elements
		map <int, double> doublearg;
        
		/**
			Set up an element, from returned query results, at a specified location
			\param res Returned query results
			\param row Row where the element is located
			\param column Column wherer the element is located
			\param type element type
		*/
		int setElement(PGresult *res,int row,int column, int type);

		/**
			Grab a double element at a specified column
			\param column Column wherer the element is located
			\param res Returned double element
			\param parameter_type Strong type of the element, the element will be cast to double when return
		*/
        int getElement(int column,double *res,int parameter_type);
		
		/**
			Grab a long element at a specified column
			\param column Column wherer the element is located
			\param res Returned long element
			\param parameter_type Strong type of the element, the element will be cast to long when return
		*/
        int getElement(int column,long *res,int parameter_type);
		
		/**
			Grab a string element at a specified column
			\param column Column wherer the element is located
			\param res Returned string element
		*/
        int getElement(int column,string *res);
		
		/**
			Grab a strong double element at a specified column
			\param column Column wherer the element is located
			\param res Returned double element
		*/
		int getElementStrongType(int column,double *res);
		
		/**
			Grab a strong long element at a specified column
			\param column Column wherer the element is located
			\param res Returned double element
		*/
        int getElementStrongType(int column,long *res);
		
		/**
			
		*/
	};

	/**
	An implementation of Outcome interfaces.
	*/
	class ResultSet: public virtual Outcome{
		public:
		ResultSet();
		ResultSet(const ResultSet& other);
		~ResultSet();
		
		int setElements(PGresult *res);// res type is decided by the DBMS, e.g., PGresult in PostgreSQL)
		int getNumberOfRows();
		int getNumberOfColumns();
		int getElement(int row, int column, long *res);
		int getElement(int row, int column, double *res);
		int getElement(int row, int column, string *res);
		int getElement(int row,int column,void *res);
		
		int getElement(int row,string name,double *res);    
		int getElement(int row,string name,long *res);
		int getElement(int row,string name,string *res);
		int getElement(int row,string name,void *res);
		
		ColumnData getARow(int row);
		void addARow(ColumnData aRow);
		
		/**
			Get column names of the query results
		*/
		map<string, int> getColumnName(){return column_name;};
		
		/**
			Get parameter types of the query results
		*/
		int* getParameterTypes(){return parameter_type;};
		
		/**
			Set column names of the query results
		*/
		void  setColumnName(map<string,int> colName);
		
		/**
			Set parameter types of the query results
		*/
		void setParamTypes( int* paramType, int colCount);
		
		/**
			Clear up internal data before filled in new result data. Must call before using the object.
		*/
		void clear();	
		
		/**
		Set parameter names from returned query results
		*/
		
		int setParameterName(PGresult *res);
		
		/**
		Set parameter types from returned query results
		*/
		int setParameterType(PGresult *res);
	
		ResultSet &operator=(ResultSet const &other);

		/// Store query data in a vector of ColumnData
		vector<ColumnData> QueryData;
		
		/// store returned query data types
		int* parameter_type; 
		int reserved;
		
		/// a map of column name to index
		map<string,int> column_name;
		
		/// number of rows
		int rows;
		
		/// number of columns
		int columns;
		
	};


}

#endif


