/**
 * \file ParseDBConfig.hxx implements an interface to read in PerfDMF configuration file and parse the parameters.
 * \author Li Li
 * \date 4/30/08
 */

#ifndef PARSEDBCONFIG_HXX_
#define PARSEDBCONFIG_HXX_

#include <sstream>
#include <fstream>
#include <map>
#include "Utility.hxx"

using namespace std;

namespace cqosdb {
	
	/**
		A class intended to read in PerfDMF configuration file and parse parameter.
	*/
class ConfigFile {

protected:
	/** 
		separator between key and value
	*/
	string myDelimiter; 
	
	/**
		indicator of comments
	*/
	string myComment;    
	
	/**
		extracted keys and values
	*/		
	map<string,string> myContents;  
	
	typedef map<string,string>::iterator mapi;
	typedef map<string,string>::const_iterator mapci;

public:

	/**
		Construct a ConfigFile, getting keys and values from given file
	*/
	ConfigFile( string filename,
	            string delimiter = ":",
	            string comment = "#");
				
	ConfigFile();
	
	/**
		Read the value corresponding to key 
	*/	
	template<class T> T read( const string& key ) const;  // call as read<T>
	
	/**
		Return the value corresponding to key or a given default value as specified by 'value' if key is not found
	*/
	template<class T> T read( const string& key, const T& value ) const;
	
	/**
		Get the value corresponding to key and store in var, Return true if key is found. 
		Otherwise leave var untouched
	*/
	template<class T> bool readInto( T& var, const string& key ) const;
	
	/**
		Get the value corresponding to key and store in var. 
		Return true if key is found
		Otherwise set var to given default value in 'value'
	*/
	template<class T> bool readInto( T& var, const string& key, const T& value ) const;
	
	/**
		Add keys and values
	*/
	template<class T> void add( string key, const T& value );
	
	/**
		Delete key and value
	*/
	void remove( const string& key );
	
	/** 
		Check whether a key exists in configuration
	*/
	bool keyExists( const string& key ) const;
	
	/**
	 Grab separator between key and value
	*/
	string getDelimiter(){ return myDelimiter; }
	
	/**
	  Grab insicator of comment
	*/
	string getComment() { return myComment; }

	/** 
		Set separator between key and value
	*/
	void setDelimiter( const string& s )
		{ string myDelimiter = s; }  
		
	/**
		Set indicator of comment
	*/
	void setComment( const string& s )
		{ string myComment = s; }
	
	/** 
		Save a ConfigFile to os
	*/
	friend ostream& operator<<( ostream& os, const ConfigFile& cf );
	
	/** 
		Load a ConfigFile from is, Read in keys and values
	*/
	friend istream& operator>>( istream& is, ConfigFile& cf );
	
public:
	/**
		File not found exception type
	*/
	struct file_not_found {
		string filename;
		file_not_found( const string& filename_ = string() )
			: filename(filename_) {} 
	};
			
	/**
		Key not found exception type, thrown only by T read(key) variant of read()
	*/
	struct key_not_found { 
		string key;
		key_not_found( const string& key_ = string() )
			: key(key_) {} };
};


template<class T>
T ConfigFile::read( const string& key ) const
{
	mapci p = myContents.find(key);
	if( p == myContents.end() ) throw key_not_found(key);
	return Utility::string_as_T<T>( p->second );
}


template<class T>
T ConfigFile::read( const string& key, const T& value ) const
{
	mapci p = myContents.find(key);
	if( p == myContents.end() ) return value;
	return Utility::string_as_T<T>( p->second );
}


template<class T>
bool ConfigFile::readInto( T& var, const string& key ) const
{
	mapci p = myContents.find(key);
	bool found = ( p != myContents.end() );
	if( found ) var = Utility::string_as_T<T>( p->second );
	return found;
}


template<class T>
bool ConfigFile::readInto( T& var, const string& key, const T& value ) const
{
	mapci p = myContents.find(key);
	bool found = ( p != myContents.end() );
	if( found )
		var = Utility::string_as_T<T>( p->second );
	else
		var = value;
	return found;
}


template<class T>
void ConfigFile::add( string key, const T& value )
{
	
	string v = Utility::T_as_string( value );
	Utility::trim(key);
	Utility::trim(v);
	myContents[key] = v;
	return;
}

}

#endif


